<?php

namespace App;
use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Auth;
class Job extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the Area that owns the job.
     */
    public function area()
    {
        return $this->belongsTo('App\Area');
    }

    /**
     * Get the Sales Officer that owns the job.
     */
    public function user()
    {
        return $this->belongsTo('App\User','user_id','id');
    }
    /**
     * Get the job_detail for the job.
     */
    public function job_details()
    {
        return $this->hasMany('App\JobDetail','job_id','id');
    }
    /***
     * List jobs
     *
     * @return mixed
     */
    public function listJobs(Request $request)
    {
        $data = $this::where('agency_id',Auth::user()->agency_id)->orderBy('id', 'Asc');
        if ($request->sale_officer != '') {
            $data->where('user_id', '=', $request->sale_officer);
        }
        if ($request->search != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->where('area_id', '=',$request->search);
            });
        }
        if ($request->search2 != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->where('user_id', '=',$request->search2);
            });
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update Job
     *
     * @return mixed
     */
    public function InsertOrUpdateJob(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $job=$this->findOrFail($id);
            $new=false;
        }else{
            $job=new $this();
            $job->uuid= generate_uuid();
            $job->end_date= $request->input('end_date');
        }
        $job->area_id= $request->input('area');
        $job->agency_id =  Auth::user()->agency_id;
        $job->user_id= $request->input('sale_officer');
        $job->start_date= $request->input('start_date');
        $job->status= $request->input('status');
        $job->comment= $request->input('comment');
        $job->save();
        if($new==true){
            $area=Area::where('id',$request->input('area'))->with(['area_revisit'])->first();
            if(isset($area['area_revisit']) && count($area['area_revisit'])>0){
                $date1=date_create($request->input('start_date'));
                $date2=date_create($request->input('end_date'));
                $diff=date_diff($date1,$date2);
                $dif=(int) $diff->format("%a");
                if(!empty($dif)  && $dif>0){
                    if($area['area_revisit'][0]['revisit_type_id']==1){
                        for($a=0;$a<=$dif;$a++){
                            $date1->modify('+'.$area['area_revisit'][0]['value'].' day');
                                $job1=new $this();
                                $job1->uuid= generate_uuid();
                                $job1->area_id= $request->input('area');
                                $job1->agency_id =  Auth::user()->agency_id;
                                $job1->user_id= $request->input('sale_officer');
                                $job1->start_date= $date1->format('Y-m-d');
                                $job1->end_date= $request->input('end_date');
                                $job1->status= $request->input('status');
                                $job1->comment= $request->input('comment');
                                $job1->save();
                            $dif=(int) $dif-(int)$area['area_revisit'][0]['value'];
                        }
                    }elseif($area['area_revisit'][0]['revisit_type_id']==2){
                        if(count($area['area_revisit'])>0){
                            $days=[];
                            foreach ($area['area_revisit'] as $revisit){
                                $days[]=$revisit['value'];
                            }
                            for($a=0;$a<=$dif;$a++){
                                $date1->modify('+1 day');
                                $timestamp = strtotime($date1->format('Y-m-d'));
                                $day = date('l', $timestamp);
                                if (in_array($day, $days))
                                {
                                    $job1=new $this();
                                    $job1->uuid= generate_uuid();
                                    $job1->area_id= $request->input('area');
                                    $job1->agency_id =  Auth::user()->agency_id;
                                    $job1->user_id= $request->input('sale_officer');
                                    $job1->start_date= $date1->format('Y-m-d');
                                    $job1->end_date= $request->input('end_date');
                                    $job1->status= $request->input('status');
                                    $job1->comment= $request->input('comment');
                                    $job1->save();
                                }
                                if($date1->format('Y-m-d')==$date2->format('Y-m-d'))
                                    $a=$dif;
                            }
                        }
                    }elseif ($area['area_revisit'][0]['revisit_type_id']==3){
                        if (count($area['area_revisit']) > 0) {
                            $dates = [];
                            foreach ($area['area_revisit'] as $revisit) {
                                $date3=date_create($revisit['value']);
                                $dates[] = $date3->format('d');
                            }
                            sort($dates);
                            array_unique($dates);
                            for($a=0;$a<=$dif;$a++){
                                $date1->modify('+1 day');
                                if (in_array($date1->format('d'), $dates))
                                {
                                    $job1=new $this();
                                    $job1->uuid= generate_uuid();
                                    $job1->area_id= $request->input('area');
                                    $job1->agency_id =  Auth::user()->agency_id;
                                    $job1->user_id= $request->input('sale_officer');
                                    $job1->start_date= $date1->format('Y-m-d');
                                    $job1->end_date= $request->input('end_date');
                                    $job1->status= $request->input('status');
                                    $job1->comment= $request->input('comment');
                                    $job1->save();
                                }
                                if($date1->format('Y-m-d')==$date2->format('Y-m-d'))
                                    $a=$dif;
                            }

                        }

                    }

                }
            }
        }
    }
    function date_sort($a, $b) {
        return strtotime($a) - strtotime($b);
    }
}
