<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Auth;
use Illuminate\Database\Eloquent\SoftDeletes;

class Shop extends Model
{
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];
    /**
     * Get the saleOfficer that owns the shop.
     */
    public function saleOfficer()
    {
        return $this->belongsTo('App\User','sale_officer_id','id');
    }
    /**
     * Get the proprietor that owns the shop.
     */
    public function proprietor()
    {
        return $this->belongsTo('App\User','proprietor_id');
    }
    /**
    * Get the proprietor that owns the shop.
    */
    public function contact_person()
    {
        return $this->belongsTo('App\User','contact_person_id');
    }
    /**
     * Get the region that owns the shop.
     */
    public function region()
    {
        return $this->belongsTo('App\Region','region_id');
    }
    /**
     * Get the region that owns the shop.
     */
    public function currency()
    {
        return $this->belongsTo('App\Currency','currency_id');
    }
    /**
     * Get the stock of the shop.
     */
    public function stock()
    {
        return $this->hasMany('App\ShopStock','shop_id');
    }
    public function shopImages() {

        return $this->hasMany('App\ShopImage');
    }
    /***
     * Area Revisit  Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function product_discount()
    {
        return $this->hasMany('App\ShopProductDiscount', 'shop_id');
    }
    // shop images
    public function shopOtherImages($id)
    {
        return ShopImage::where('type','Other')->where('shop_id',$id)->get();
    }
    // shop images
    public function shopDisplayImages($id)
    {
        return ShopImage::where('type','Display')->where('shop_id',$id)->get();
    }

    /***
     * List Shops
     *
     * @return mixed
     */
    public function listShops(Request $request)
    {
        $data = $this::where('agency_id',Auth::user()->agency_id)->orderBy('id', 'Desc');
        if ($request->search != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('id', '=',$request->search);
            });
        }
        if ($request->sale_officer != '') {
            $data->where('sale_officer_id', '=', $request->sale_officer);
        }

        if ($request->country != '') {
            $currency=array();
            $all_currency=Currency::select(['id','name'])->where('country_id',$request->country)->get();
            if(count($all_currency)>0){
                foreach ($all_currency as $key=>$cur){
                    $currency[]=$cur['id'];
                }
            }
            if(count($currency)>0)
            $data->whereIn('currency_id', $currency);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert or Update Shop
     *
     * @return mixed
     */
    public function InsertOrUpdateShop(Request $request,$id=0)
    {
        $new=0;
        if($id>0){
            $shop=$this->findOrFail($id);
            $new=1;
            $shop->updated_by = Auth::user()->id;
        }else{
            $shop=new $this();
            $shop->created_by = Auth::user()->id;
            $shop->uuid=generate_uuid();
            if(!empty($request->input('opening_balance'))){
            $shop->opening_balance= $request->input('opening_balance');
            $shop->balance=(-1)* $request->input('opening_balance');
            }

            // add point on first shop add
           $so_point=new SoPoint();
           $so_point->user_id= $request->input('sale_officer');
           $so_point->point_type_id= 1;
           $so_point->date= date('Y-m-d');
           $so_point->save();
        }
        //Proprietor ID
        if(array_key_exists('proprietor', $request->all())) {
            $userModel = new User();
            $shop->proprietor_id = $userModel->createOrUpdateUser($request->input('proprietor'));
            if($request->input('same_as_above')){
                $shop->contact_person_id=$shop->proprietor_id;
            }
        }
        //Contact Person ID
        if(!$request->input('same_as_above')){
            if(array_key_exists('contact_person', $request->all())) {
                $userModel = new User();
                $shop->contact_person_id = $userModel->createOrUpdateUser($request->input('contact_person'));
            }
        }

        $shop->parent_id = Auth::user()->id;
        $shop->agency_id =  Auth::user()->agency_id;
        $shop->name= $request->input('name');
        $shop->email= $request->input('email');
        $shop->location= $request->input('location');
        $shop->latitude= $request->input('latitude');
        $shop->longitude= $request->input('longitude');
        $shop->phone_1= $request->input('phone_1');
        $shop->phone_2= $request->input('phone_2');
        $shop->landline= $request->input('landline');
        $shop->discount_percentage= $request->input('discount_percentage');
        $shop->biocos_ratting= $request->input('biocos_ratting');
        $shop->area_id= $request->input('area');
        $shop->currency_id= $request->input('currency');
        $shop->status= $request->input('status');
        $shop->sale_officer_id= $request->input('sale_officer');
        $shop->customer_type= $request->input('customer_type');
        if($request->input('customer_type')=='Distributer'){
            $shop->sale_officer_id=0;
            $shop->area_id=0;
        }
        if($request->input('customer_type')=='Retail Saler'){
            $shop->shop_type= $request->input('shop_type');
        }else{
            $shop->shop_type= null;
        }
        $shop->Category= $request->input('Category');
        $shop->target_visit_id= $request->input('target_visit');
        if($shop->save()){

            // Shop Image added
            if($new==1){
                $old_image_id=[];
                $all_product=ShopImage::select('id')->where('shop_id',$shop->id)->get()->toarray();
                if(count($all_product)>0){
                    foreach ($all_product as $all){
                        if(isset($all['id']) && !empty($all['id'])){
                            $old_image_id[]=$all['id'];
                        }
                    }
                }
                if(count($request->input('old_images_id'))>0){
                    foreach ($request->input('old_images_id') as $value){
                        if (($key = array_search($value, $old_image_id)) !== FALSE) {
                            unset($old_image_id[$key]);
                        }
                    }
                }
                if(count($old_image_id)>0){
                    foreach ($old_image_id as $id){
                        $old_order_product=ShopImage::findOrFail($id);
                        $old_order_product->delete();
                        $file = public_path() .str_replace(url(''),"",$old_order_product['image_url']);
                        if (file_exists($file)) {
                            @unlink($file);
                        }
                    }
                }
            }
            for($i=-1,$j=0;$i<12;$i++,$j++){
                if ($request->hasFile('image'.$j)) {
                    $filename = 'image-' . time() . rand(99, 199) . '.' . $request->file('image'.$j)->getClientOriginalExtension();
                    $request->file('image'.$j)->move(config('constants.UPLOAD_PATH') . 'shops/', $filename);
                    if (isset($filename)){
                        $pro_image=new ShopImage();
                        $pro_image->image_url = url('/uploads/shops/'.$filename);
                        $pro_image->shop_id = $shop->id;
                        $pro_image->save();
                    }
                }
            }

            // Shop Product Grade added
            if(count($request->input('products'))>0){
                $shop_discount_id=[];
                $ShopProductDiscount=ShopProductDiscount::select('id')->where('shop_id',$shop->id)->get()->toarray();
                foreach ($ShopProductDiscount as $discount){
                    if(isset($discount['id']) && !empty($discount['id'])){
                        $shop_discount_id[]=$discount['id'];
                    }
                }

                foreach ($request->input('products') as $value){
                    if(isset($value['old_customer_discount_id']) && !empty($value['old_customer_discount_id'])){
                        if (($key = array_search($value['old_customer_discount_id'], $shop_discount_id)) !== FALSE) {
                            unset($shop_discount_id[$key]);
                        }
                        $shop_discount=ShopProductDiscount::findOrFail($value['old_customer_discount_id']);
                    }else{
                        $shop_discount=new ShopProductDiscount();
                    }
                    $shop_discount->shop_id=$shop->id;
                    $shop_discount->product_id=$value['product_id'];
                    if($request->input('currency')==1)
                    $shop_discount->product_discount_id=$value['discount_id_pkr'];
                    if($request->input('currency')==2){
                        if($request->input('customer_type')=="Retail Saler")
                        $shop_discount->product_discount_id=$value['discount_id_AED_retail_price'];
                        if($request->input('customer_type')=="Wholesaler")
                            $shop_discount->product_discount_id=$value['discount_id_AED_wholesale_price'];
                    }
                    $shop_discount->save();
                }
                if(count($shop_discount_id)>0){
                    foreach ($shop_discount_id as $id){
                        $old_shop_discount=ShopProductDiscount::findOrFail($id);
                        $old_shop_discount->delete();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
