<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;

class SoTarget extends Model
{
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [ 'id' ];

    /***
     * SO Target Detail Relationship
     *
     * @return \Illuminate\Database\Eloquent\Relations\hasMany
     */
    public function target_details()
    {
        return $this->hasMany('App\SoTargetDetail', 'so_target_id');
    }
    /**
     * Get the user that owns the target.
     */
    public function user()
    {
        return $this->belongsTo('App\User','so_id','id');
    }
    public function product_target($id,$so_target_id)
    {
        $target= SoTargetDetail::where('product_id',$id)->where('so_target_id',$so_target_id)->first();
        if(!empty($target)){
            return $target['target']/12;
        }
        return "";
    }
    public function Achievedtarget($product_id,$date,$so_id){
        $orders=Order::select('id')
            ->where('booked_by',$so_id)
            ->where('status','Delivered')
//            ->whereBetween('date_to_deliver', ['2018-06-01','2018-06-30'])
            ->where('date_to_deliver','like', '%' . $date . '%')
            ->where('type','!=','Sample')
            ->get();
        if(count($orders)>0){
            $sum=0;

            foreach ($orders as $order){
                $quantity=0;
                $quantity=OrderProduct::where('order_id',$order['id'])->where('product_id',$product_id)->sum('quantity');
                $sum=$sum+$quantity;
            }

            return $sum;
        }
        return '';
    }


    /***
     * List Target
     *
     * @return mixed
     */
    public function listTargets(Request $request)
    {
        $data = $this::orderBy('id', 'Desc');
        if ($request->search_name != '') {
            $data->where(function ($query) use ($request) {
                return $query
                    ->Where('date', 'like', '%' . $request->search_name . '%');
            });
        }
        if ($request->sale_officer != '') {
            $data->where('so_id', '=', $request->sale_officer);
        }
        return $data->paginate(session()->get('site_settings')['results_per_page']);
    }
    /***
     * Insert/Update
     *
     * @return mixed
     */
    public function InsertOrUpdate(Request $request,$id=0)
    {
        $new=true;
        if($id>0){
            $target=$this->findOrFail($id);
            $new=false;
        }else{
            $target=new $this();
        }
        $target->so_id= $request->input('sale_officer');
        $target->date= $request->input('date');
        if($request->input('productwise')){
            $target->productwise= true;
        }else{
            $target->productwise= false;
        }
        if($request->input('customer_add')){
            $target->customer_add_target= $request->input('customer_add_target');
            $target->customer_add= true;
        }else{
            $target->customer_add_target= null;
            $target->customer_add= false;
        }
        if($request->input('customer_visit')){
            $target->customer_visit_target= $request->input('customer_visit_target');
            $target->customer_visit= true;
        }else{
            $target->customer_visit_target= null;
            $target->customer_visit= false;
        }
        if($target->save()){
            //So target detail added
            SoTargetDetail::where('so_target_id',$target->id)->delete();
            if($request->input('productwise')){
                foreach ($request->input('products') as $product){
                    if(!empty($product['target'])){
                        $detail=new SoTargetDetail();
                        $detail->so_target_id=$target->id;
                        $detail->product_id=$product['id'];
                        $detail->target=$product['target']*12;
                        $detail->save();
                    }
                }
            }
            return true;
        }else{
            return false;
        }
    }
}
